import random
import unittest
import duolingo
import urlparse
import httplib2
import urllib
import json

CONSUMER_KEY = ''
CONSUMER_SECRET = ''

ACCESS_KEY = ''
ACCESS_SECRET = ''

USERNAME = ''
PASSWORD = ''

LOGIN_URL = 'http://localhost:8080/login'
ALLOW_URL = 'http://localhost:8080/oauth/allow'
REVOKE_URL = 'http://localhost:8080/oauth/revoke'

class TestAuthorization(unittest.TestCase):
    """tests the oauth authorization"""

    def setUp(self):
        self.duolingo = duolingo.Duolingo(consumer_key=CONSUMER_KEY, consumer_secret=CONSUMER_SECRET)
        self.http = httplib2.Http()

    def tearDown(self):
        pass

    def test_initialization(self):
        self.assertRaises(duolingo.BadInitializationError, duolingo.Duolingo, consumer_key='erw', access_secret='jker')
        self.assertRaises(duolingo.BadInitializationError, duolingo.Duolingo, consumer_secret='erw', access_secret='jker')

    def test_request_token(self):
        self.duolingo.request_token()

    @unittest.skip
    def test_flow(self):
        #get request token
        url = self.duolingo.request_token()
        oauth_token = url.split('oauth_token')[1][1:]
        #login as user and authorize application
        self.login()
        verifier = self.allow(oauth_token)
        #get access token
        self.duolingo.access_token(verifier)
        #delete installation
        self.revoke(self.duolingo._access_token.key)

    def login(self):
        body = {'login': USERNAME, 'password': PASSWORD}
        headers = {'Content-type': 'application/x-www-form-urlencoded'}
        response, content = self.http.request(LOGIN_URL, 'POST', headers=headers, body=urllib.urlencode(body))
        self.cookie = response['set-cookie']

    def allow(self, oauth_token):
        body = {'oauth_token': oauth_token}
        headers = {'Content-type': 'application/x-www-form-urlencoded', 'Cookie': self.cookie}
        response, content = self.http.request(ALLOW_URL, 'POST', headers=headers, body=urllib.urlencode(body))
        redirect = json.loads(content)['redirect']
        verifier = dict(urlparse.parse_qsl(urlparse.urlparse(redirect).query))['verifier']
        return verifier

    def revoke(self, oauth_token):
        body = {'oauth_token': oauth_token}
        headers = {'Content-type': 'application/x-www-form-urlencoded', 'Cookie': self.cookie}
        response, content = self.http.request(REVOKE_URL, 'POST', headers=headers, body=urllib.urlencode(body))

class TestDuolingo(unittest.TestCase):
    """tests the actual duolingo API calls."""

    def setUp(self):
        self.duolingo = duolingo.Duolingo(consumer_key=CONSUMER_KEY, consumer_secret=CONSUMER_SECRET, access_key=ACCESS_KEY, access_secret=ACCESS_SECRET)

    def tearDown(self):
        pass

    def test_create_translate_job(self):
        job = self.duolingo.create_translate_job({
            u'title': u'Hello',
            u'text': u'erfsssafs',
            u'source_language': u'en',
            u'target_language': u'es',
            u'license': u'cc',
        })
        self.assertEquals('submitted', job['state'])

    def test_get_translate_job(self):
        #create translate job
        job = self.duolingo.create_translate_job({
            u'title': u'Hello',
            u'text': u'erfsssafs',
            u'source_language': u'en',
            u'target_language': u'es',
            u'license': u'cc',
        })
        #get it again
        job2 = self.duolingo.get_translate_job(job['id'])
        self.assertEquals(job2, job)

    def test_cancel_job(self):
        job = self.duolingo.create_translate_job({
            u'title': u'Hello',
            u'text': u'erfsssafs',
            u'source_language': u'en',
            u'target_language': u'es',
            u'license': u'cc',
        })
        job.cancel()
        self.assertEquals('cancelled', job['state'])

    @unittest.skip
    def test_approve_job(self):
        job = self.duolingo.create_translate_job({
            u'title': u'Hello',
            u'text': u'erfsssafs',
            u'source_language': u'en',
            u'target_language': u'es',
            u'license': u'cc',
        })
        job.approve()
        self.assertEquals('approved', job['state'])

    @unittest.skip
    def test_reject_job(self):
        job = self.duolingo.create_translate_job({
            u'title': u'Hello',
            u'text': u'erfsssafs',
            u'source_language': u'en',
            u'target_language': u'es',
            u'license': u'cc',
        })
        job.reject('not a good translation')
        self.assertEquals('rejected', job['state'])

if __name__ == '__main__':
    unittest.main()


from distutils.core import setup

setup(
    name='duolingo',
    version='0.1.1',
    author='Severin Hacker',
    author_email='severin@duolingo.com',
    packages=['duolingo'],
    scripts=[],
    url='http://duolingo.com',
    license='LICENSE.txt',
    description='python client for the duolingo API',
    long_description=open('README.txt').read(),
    install_requires=[
        "oauth2 >= 1.5.211",
        "httplib2 >= 0.7.2",
    ],
)


import urlparse
import oauth2 as oauth
import json
import httplib
import urllib

class BadInitializationError(Exception):
    pass

class BadStateError(Exception):
    pass

class AuthenticationError(Exception):
    pass

class NotAuthorizedError(Exception):
    pass

class DuolingoError(Exception):
    pass

REQUEST_TOKEN_URL = 'http://localhost:8080/oauth/request_token'
ACCESS_TOKEN_URL = 'http://localhost:8080/oauth/access_token'
AUTHORIZE_URL = 'http://localhost:8080/#/oauth/authorize'

BASE_URL = 'http://localhost:8080/'


class Duolingo(object):

    def __init__(self, consumer_key=None, consumer_secret=None, access_key=None, access_secret=None):
        if consumer_key != None and consumer_secret != None and access_key != None and access_secret != None:
            self._access_token = oauth.Token(key=access_key, secret=access_secret)
            self.consumer = oauth.Consumer(consumer_key, consumer_secret)
            self.client = oauth.Client(self.consumer, self._access_token)
            self.state = 'authorized'
        elif consumer_key != None and consumer_secret != None:
            self.consumer = oauth.Consumer(consumer_key, consumer_secret)
            self.state = 'unauthorized'
        else:
            raise BadInitializationError("you must initialize with at least a access_key/access_secret pair")

    def request_token(self):
        if not self.state == 'unauthorized':
            raise BadStateError("you must be unauthorized to call this methoed")
        client = oauth.Client(self.consumer)
        resp, content = client.request(REQUEST_TOKEN_URL, "GET")
        if resp['status'] != '200':
            raise AuthenticationError("Invalid response %s." % resp['status'])
        request_token = dict(urlparse.parse_qsl(content))
        self.request_token = oauth.Token(key=request_token['oauth_token'], secret=request_token['oauth_token_secret'])
        self.state = 'requested'
        return "%s?oauth_token=%s" % (AUTHORIZE_URL, request_token['oauth_token'])

    def access_token(self, oauth_verifier):
        if not self.state == 'requested':
            raise BadStateError("you must have a request token first")
        self.request_token.set_verifier(oauth_verifier)
        client = oauth.Client(self.consumer, self.request_token)
        resp, content = client.request(ACCESS_TOKEN_URL, "POST")
        access_token = dict(urlparse.parse_qsl(content))
        self._access_token = oauth.Token(key=access_token['oauth_token'], secret=access_token['oauth_token_secret'])
        self.client = oauth.Client(self.consumer, self._access_token)
        self.state = 'authorized'

    def __access_resource(self, url, method='GET', parameters={}, values={}):
        actual_url = url.replace('{{id}}', str(values['id'])) if 'id' in values else url
        if len(parameters)==0:
            resp, content = self.client.request(actual_url, method=method)
        else:
            resp, content = self.client.request(actual_url, method=method, body=urllib.urlencode(parameters))
        if resp['status'] != '200':
            raise DuolingoError("could not retrieve job:"+content)
        else:    
            data = json.loads(content)
            if data == None:
                raise NotAuthorizedError()
            if len(data) == 0:
                raise NotAuthorizedError()
            return data

    def create_translate_job(self, job, sandbox=False):
        return TranslateJob(self, self.__access_resource(BASE_URL+'translate_jobs', method="POST", parameters={'model':json.dumps(job)}))

    def get_translate_job(self, id_):
        return TranslateJob(self, self.__access_resource(BASE_URL+'translate_jobs/{{id}}', values={'id':id_}))

class TranslateJob(dict):
    
    def __init__(self, duolingo, data={}):
        self.duolingo = duolingo
        self.update(data)

    def __call(self, url, parameters={}):
        data = self.duolingo.__access_resource(url, method='POST', parameters=parameters, values={'id': self['id']})
        self.update(data)

    def cancel(self):
        return self.__call(BASE_URL+'translate_jobs/{{id}}/cancel')

    def approve(self):
        return self.__call(BASE_URL+'translate_jobs/{{id}}/approve')
        
    def reject(self, reason):
        return self.__call(BASE_URL+'translate_jobs/{{id}}/reject', {'reason': reason})